@@ -6,31 +6,26 @@
 static int min_JM (int *, int *);
 static int max_JM (int *, int *);
 
-static void rowcentre_JM (float *, int, int);
-static void colstandard_JM (float *, int, int);
-static void rowstd_JM (float *, int, int, int);
-static void transpose_mat_JM (float *, int *, int *, float *);
-static void mmult_JM (float *, int, int, float *, int, int, float *);
-static void orthog_mat_JM (float *, int, float *);
-static void gramsch_JM (float *, int, int, int);
-static void svd_JM (float *, int *, int *, float *, float *, float *);
+static void rowcentre_JM (double *, int, int);
+static void colstandard_JM (double *, int, int);
+static void rowstd_JM (double *, int, int, int);
+static void transpose_mat_JM (double *, int *, int *, double *);
+static void mmult_JM (double *, int, int, double *, int, int, double *);
+static void orthog_mat_JM (double *, int, double *);
+static void gramsch_JM (double *, int, int, int);
+static void svd_JM (double *, int *, int *, double *, double *, double *);
 
-static void Symm_logcosh_JM (float *, int, float *, int, int, float, float *, float *);
-static void Symm_exp_JM (float *, int, float *, int, int, float, float *, float *);
-static void Def_logcosh_JM (float *, int, float *, int, int, float, float *);
-static void Def_exp_JM (float *, int, float *, int, int, float, float *);
-static void calc_A_JM(float*, float*, float*, int*, int*, int*, float*, float*);
-static void calc_K_JM(float*, int*, int*, float*);
-
-void F77_NAME (sgesdd) (char *, int *, int *, float *, int *, float *,
-			float *, int *, float *, int *, float *, int *,
-			int *, int *);
-void F77_NAME (sgemm) (char *, char *, int *, int *, int *, float *, float *,
-		       int *, float *, int *, float *, float *, int *);
+static void Symm_logcosh_JM (double *, int, double *, int, int, double, double *, double *);
+static void Symm_exp_JM (double *, int, double *, int, int, double, double *, double *);
+static void Def_logcosh_JM (double *, int, double *, int, int, double, double *);
+static void Def_exp_JM (double *, int, double *, int, int, double, double *);
+static void calc_A_JM(double*, double*, double*, int*, int*, int*, double*, double*);
+static void calc_K_JM(double*, int*, int*, double*);
 
+#include <R_ext/Lapack.h>
 
 static void
-rowcentre_JM (float *ans, int n, int p)
+rowcentre_JM (double *ans, int n, int p)
 {
 /*  mean centres nxp matrix ans */
     double tmp;
@@ -41,13 +36,13 @@ rowcentre_JM (float *ans, int n, int p)
 	    tmp = tmp + ((double) ans[p * i + j]) / p;
 	}
 	for (j = 0; j < p; j++) {
-	    ans[p * i + j] -= (float) tmp;
+	    ans[p * i + j] -= (double) tmp;
 	}
     }
 }
 
 static void
-colstandard_JM (float *ans, int n, int p)
+colstandard_JM (double *ans, int n, int p)
 {
 /*  transform columns of nxp matrix ans to have zero mean and unit variance */
     double tmp[2];
@@ -68,28 +63,28 @@ colstandard_JM (float *ans, int n, int p)
 	tmp[1] = sqrt (tmp1);
 	for (j = 0; j < n; j++) {
 	    ans[p * j + i] =
-		(float) ((((double) ans[p * j + i]) - tmp[0]) / tmp[1]);
+		(double) ((((double) ans[p * j + i]) - tmp[0]) / tmp[1]);
 	}
     }
 }
 
 
 static void
-svd_JM (float *mat, int *n, int *p, float *u, float *d, float *v)
+svd_JM (double *mat, int *n, int *p, double *u, double *d, double *v)
 {
 
     /*  calculates svd decomposition of nxp matrix mat */
-    /*    mat is a pointer to an nxp array of floats */
+    /*    mat is a pointer to an nxp array of doubles */
     /*    n is a pointer to an integer specifying the no. of rows of mat */
     /*    p is a pointer to an integer specifying the no. of cols of mat */
-    /*    u is a pointer to a float array of dimension (n,n) */
-    /*    d is a pointer to a float array of dimension min(n,p) */
-    /*    v is a pointer to a float array of dimension (p,p) */
+    /*    u is a pointer to a double array of dimension (n,n) */
+    /*    d is a pointer to a double array of dimension min(n,p) */
+    /*    v is a pointer to a double array of dimension (p,p) */
 
 
     int info, *iwork, lwork, a, b;
     size_t iwork_size, ilwork, nn = *n, pp = *p, mm;
-    float *work, *mat1, *u1, *v1;
+    double *work, *mat1, *u1, *v1;
     char jobz = 'A';
 
     mm = min_JM(n,p);
@@ -101,16 +96,16 @@ svd_JM (float *mat, int *n, int *p, float *u, float *d, float *v)
     if (ilwork > INT_MAX)
 	error("svd on %d x %d exceeds Fortran indexing limits");
 
-    work = Calloc (ilwork, float);
+    work = Calloc (ilwork, double);
     iwork = Calloc (iwork_size, int);
-    mat1 = Calloc (nn * pp, float);
-    u1 = Calloc (nn * nn, float);
-    v1 = Calloc (pp * pp, float);
+    mat1 = Calloc (nn * pp, double);
+    u1 = Calloc (nn * nn, double);
+    v1 = Calloc (pp * pp, double);
 
     transpose_mat_JM (mat, n, p, mat1);
 
     lwork = ilwork;
-    F77_CALL (sgesdd) (&jobz, n, p, mat1, n, d, u1, n, v1, p, work,
+    F77_CALL (dgesdd) (&jobz, n, p, mat1, n, d, u1, n, v1, p, work,
 		       &lwork, iwork, &info);
 
     transpose_mat_JM (u1, n, n, u);
@@ -125,7 +120,7 @@ svd_JM (float *mat, int *n, int *p, float *u, float *d, float *v)
 
 
 static void
-transpose_mat_JM (float *mat, int *n, int *p, float *ans)
+transpose_mat_JM (double *mat, int *n, int *p, double *ans)
 {
 /*    transpose nxp matrix mat */
     int i, j;
@@ -163,12 +158,12 @@ static int max_JM (int *a, int *b)
 
 
 static void
-mmult_JM (float *A, int n, int p, float *B, int q, int r, float *C)
+mmult_JM (double *A, int n, int p, double *B, int q, int r, double *C)
 {
 /*    matrix multiplication using FORTRAN BLAS routine SGEMM */
 /*    A is (n*p) and B is (q*r), A*B returned to C  */
 
-    float alpha = 1.0, beta = 0.0;
+    double alpha = 1.0, beta = 0.0;
     int M, K, N;
     char transA = 'N', transB = 'N';
 
@@ -178,24 +173,24 @@ mmult_JM (float *A, int n, int p, float *B, int q, int r, float *C)
 	M = n;
 	K = p;
 	N = r;
-	F77_CALL (sgemm) (&transA, &transB, &N, &M, &K, &alpha, B, &N,
+	F77_CALL (dgemm) (&transA, &transB, &N, &M, &K, &alpha, B, &N,
 			  A, &K, &beta, C, &N);
     }
 }
 
 static void
-orthog_mat_JM (float *mat, int e, float *orthog)
+orthog_mat_JM (double *mat, int e, double *orthog)
 {
 	/* take Wmat, (e*e), and return orthogonalized version to orthog_W */
-	float *u, *v, *d, *temp;
+	double *u, *v, *d, *temp;
 	int i;
 	size_t ee = e;
 
 
-	u = Calloc (ee * ee, float);
-	d = Calloc (ee, float);
-	v = Calloc (ee * ee, float);
-	temp = Calloc (ee * ee, float);
+	u = Calloc (ee * ee, double);
+	d = Calloc (ee, double);
+	v = Calloc (ee * ee, double);
+	temp = Calloc (ee * ee, double);
 
 	svd_JM (mat, &e, &e, u, d, v);
 	for (i = 0; i < e; i++) {
@@ -216,14 +211,14 @@ orthog_mat_JM (float *mat, int e, float *orthog)
 }
 
 static void
-Symm_logcosh_JM (float *w_init, int e, float *data, int f, int p, float alpha, float *w_final, float *Tol)
+Symm_logcosh_JM (double *w_init, int e, double *data, int f, int p, double alpha, double *w_final, double *Tol)
 {
 
 	/* Function that carries out Symmetric ICA using a logcosh approximation to the neg. entropy function */
 
-	float *mat1, *mat2, *mat3, *mat4, *mat5, *mat6;
+	double *mat1, *mat2, *mat3, *mat4, *mat5, *mat6;
 	int i, j;
-	float mean;
+	double mean;
 
 	if (e != f) {
 		error ("error in Symm_logcosh_JM, dims dont match");
@@ -231,12 +226,12 @@ Symm_logcosh_JM (float *w_init, int e, float *data, int f, int p, float alpha, f
 	else {
 		size_t es = (size_t)e * (size_t)e;
 		size_t ep = (size_t)e * (size_t)p;
-		mat1 = Calloc (ep, float);
-		mat2 = Calloc (ep, float);
-		mat3 = Calloc (es, float);
-		mat4 = Calloc (es, float);
-		mat5 = Calloc (es, float);
-		mat6 = Calloc (es, float);
+		mat1 = Calloc (ep, double);
+		mat2 = Calloc (ep, double);
+		mat3 = Calloc (es, double);
+		mat4 = Calloc (es, double);
+		mat5 = Calloc (es, double);
+		mat6 = Calloc (es, double);
 
 		mmult_JM (w_init, e, e, data, e, p, mat1);
 
@@ -296,22 +291,22 @@ Symm_logcosh_JM (float *w_init, int e, float *data, int f, int p, float alpha, f
 }
 
 static void
-Def_logcosh_JM (float *w_init, int e, float *data, int f, int p, float alpha, float *w_final)
+Def_logcosh_JM (double *w_init, int e, double *data, int f, int p, double alpha, double *w_final)
 {
 /* Function that carries out Deflation ICA using an logcosh approximation to the neg. entropy function */
 
-	float *mat1, *mat2, *mat3, *mat4;
+	double *mat1, *mat2, *mat3, *mat4;
 	int i, j;
-	float mean;
+	double mean;
 
 	if (e != f) {
 		error ("error in Def_logcosh_JM, dims dont match");
 	}
 	else {
-		mat1 = Calloc (p, float);
-		mat2 = Calloc ((size_t)e * (size_t)p, float);
-		mat3 = Calloc (e, float);
-		mat4 = Calloc (e, float);
+		mat1 = Calloc (p, double);
+		mat2 = Calloc ((size_t)e * (size_t)p, double);
+		mat3 = Calloc (e, double);
+		mat4 = Calloc (e, double);
 
 		mmult_JM (w_init, 1, e, data, e, p, mat1);
 
@@ -351,13 +346,13 @@ Def_logcosh_JM (float *w_init, int e, float *data, int f, int p, float alpha, fl
 }
 
 static void
-Symm_exp_JM (float *w_init, int e, float *data, int f, int p, float alpha, float *w_final, float *Tol)
+Symm_exp_JM (double *w_init, int e, double *data, int f, int p, double alpha, double *w_final, double *Tol)
 		{
     /* Function that carries out Symmetric ICA using a exponential approximation to the neg. entropy function */
 
-float *mat1, *mat2, *mat3, *mat4, *mat5, *mat0, *mat6;
+double *mat1, *mat2, *mat3, *mat4, *mat5, *mat0, *mat6;
 int i, j;
-float mean;
+double mean;
 
 if (e != f) {
     error ("error in Symm_exp_JM, dims dont match");
@@ -365,13 +360,13 @@ if (e != f) {
 else {
     size_t ep = (size_t)e * (size_t)p;
     size_t ee = (size_t)e * (size_t)e;
-    mat0 = Calloc (ep, float);
-    mat1 = Calloc (ep, float);
-    mat2 = Calloc (ep, float);
-    mat3 = Calloc (ee, float);
-    mat4 = Calloc (ee, float);
-    mat5 = Calloc (ee, float);
-    mat6 = Calloc (ee, float);
+    mat0 = Calloc (ep, double);
+    mat1 = Calloc (ep, double);
+    mat2 = Calloc (ep, double);
+    mat3 = Calloc (ee, double);
+    mat4 = Calloc (ee, double);
+    mat5 = Calloc (ee, double);
+    mat6 = Calloc (ee, double);
     mmult_JM (w_init, e, e, data, e, p, mat1);
     for (i = 0; i < e; i++) {
 	for (j = 0; j < p; j++) {
@@ -431,22 +426,22 @@ else {
 }
 
 static void
-Def_exp_JM (float *w_init, int e, float *data, int f, int p, float alpha, float *w_final)
+Def_exp_JM (double *w_init, int e, double *data, int f, int p, double alpha, double *w_final)
 {
     /* Function that carries out Deflation ICA using an exponential approximation to the neg. entropy function */
 
-float *mat1, *mat2, *mat3, *mat4;
+double *mat1, *mat2, *mat3, *mat4;
 int i, j;
-float mean;
+double mean;
 
 if (e != f) {
     error ("error in Def_exp_JM, dims dont match");
 }
 else {
-    mat1 = Calloc (p, float);
-    mat2 = Calloc ((size_t)e * (size_t)p, float);
-    mat3 = Calloc (e, float);
-    mat4 = Calloc (e, float);
+    mat1 = Calloc (p, double);
+    mat2 = Calloc ((size_t)e * (size_t)p, double);
+    mat3 = Calloc (e, double);
+    mat4 = Calloc (e, double);
 
     mmult_JM (w_init, 1, e, data, e, p, mat1);
 
@@ -490,10 +485,10 @@ else {
 }
 
 static void
-gramsch_JM (float *ww, int n, int m, int k)
+gramsch_JM (double *ww, int n, int m, int k)
 {
 int ip, jp;
-float tmp;
+double tmp;
 /* do Gram-Schmidt on row k of (n*m) matrix ww */
 k -= 1;
 if (k > n) {
@@ -513,10 +508,10 @@ else {
 }
 
 static void
-rowstd_JM (float *ww, int n, int m, int k)
+rowstd_JM (double *ww, int n, int m, int k)
 {
 /* for ww (n*m), make ||ww[k, ]|| equal 1 */
-float tmp = 0;
+double tmp = 0;
 int i;
 k -= 1;
 if (k > n) {
@@ -535,14 +530,14 @@ else {
 
 
 static void
-calc_K_JM(float *x, int *n, int *p, float *K)
+calc_K_JM(double *x, int *n, int *p, double *K)
 {
     int i, j;
-    float *xxt, *xt, *u, *d, *v, *temp1, *temp2;
+    double *xxt, *xt, *u, *d, *v, *temp1, *temp2;
     size_t nn = *n, pp = *p;
 
-    xxt = Calloc (nn * nn, float);
-    xt = Calloc (nn * pp, float);
+    xxt = Calloc (nn * nn, double);
+    xt = Calloc (nn * pp, double);
 
     /* transpose x matrix */
     transpose_mat_JM (x, n, p, xt);
@@ -557,15 +552,15 @@ calc_K_JM(float *x, int *n, int *p, float *K)
     Free (xt);
 
     /* calculate svd decomposition of xxt */
-    u = Calloc (nn * nn, float);
-    d = Calloc (nn, float);
-    v = Calloc (nn * nn, float);
+    u = Calloc (nn * nn, double);
+    d = Calloc (nn, double);
+    v = Calloc (nn * nn, double);
 
     svd_JM (xxt, n, n, u, d, v);
 
     /* calculate K matrix*/
-    temp1 = Calloc (nn * nn, float);
-    temp2 = Calloc (nn * nn, float);
+    temp1 = Calloc (nn * nn, double);
+    temp2 = Calloc (nn * nn, double);
 
     for (i = 0; i < *n; i++) {
 	temp1[*n * i + i] = 1 / sqrt (d[i]);
@@ -583,36 +578,36 @@ calc_K_JM(float *x, int *n, int *p, float *K)
 }
 
 static void
-calc_A_JM(float *w, float *k, float *data,
-	  int *e, int *n, int *p, float *A, float *unmixed_data)
+calc_A_JM(double *w, double *k, double *data,
+	  int *e, int *n, int *p, double *A, double *unmixed_data)
 {
     /* calculate un-mixing matrix A */
     int i;
-    float *um, *umt, *umumt, *uu, *dd, *vv, *temp1, *temp2, *temp3;
+    double *um, *umt, *umumt, *uu, *dd, *vv, *temp1, *temp2, *temp3;
     size_t nn = *n, ee = *e;
 
-    um = Calloc (ee * nn, float);
-    umt = Calloc (nn * ee, float);
+    um = Calloc (ee * nn, double);
+    umt = Calloc (nn * ee, double);
 
     mmult_JM (w, *e, *e, k, *e, *n, um);
     mmult_JM (um, *e, *n, data, *n, *p, unmixed_data);
     transpose_mat_JM (um, e, n, umt);
 
-    umumt = Calloc (ee * ee, float);
+    umumt = Calloc (ee * ee, double);
     mmult_JM (um, *e, *n, umt, *n, *e, umumt);
 
-    uu = Calloc (ee * ee, float);
-    dd = Calloc (ee, float);
-    vv = Calloc (ee * ee, float);
+    uu = Calloc (ee * ee, double);
+    dd = Calloc (ee, double);
+    vv = Calloc (ee * ee, double);
     svd_JM (umumt, e, e, uu, dd, vv);
 
-    temp1 = Calloc (ee * ee, float);
+    temp1 = Calloc (ee * ee, double);
     for (i = 0; i < *e; i++) {
 	temp1[*e * i + i] = 1 / (dd[i]);
     }
 
-    temp2 = Calloc (ee * ee, float);
-    temp3 = Calloc (ee * ee, float);
+    temp2 = Calloc (ee * ee, double);
+    temp3 = Calloc (ee * ee, double);
     transpose_mat_JM (vv, e, e, temp3);
     mmult_JM (temp3, *e, *e, temp1, *e, *e, temp2);
     transpose_mat_JM (uu, e, e, vv);
@@ -633,22 +628,22 @@ calc_A_JM(float *w, float *k, float *data,
 }
 
 static void
-icainc_JM (float *data_matrix, float *w_matrix, int *nn, int *pp, int *ee,
-	float *alpha, int *rowflag, int *colflag, int *funflag, int *maxit,
-	float *lim, int *defflag, int *verbose, float *data_pre, float *Kmat1,
-	float *w_final, float *ansa, float *ansx2)
+icainc_JM (double *data_matrix, double *w_matrix, int *nn, int *pp, int *ee,
+	double *alpha, int *rowflag, int *colflag, int *funflag, int *maxit,
+	double *lim, int *defflag, int *verbose, double *data_pre, double *Kmat1,
+	double *w_final, double *ansa, double *ansx2)
 {
 
     /* main ICA function */
 
     int i, j, k;
     size_t n = *nn, p = *pp, e = *ee;
-    float tol;
-    float *temp_w1, *temp_w2;
-    float *data1, *Kmat, *temp1, *w_init;
+    double tol;
+    double *temp_w1, *temp_w2;
+    double *data1, *Kmat, *temp1, *w_init;
 
     /* make a copy of the data matrix*/
-    data1 = Calloc (n * p, float);
+    data1 = Calloc (n * p, double);
     for (i = 0; i < n; i++) {
 	for (j = 0; j < p; j++) {
 	    data_pre[i * p + j] = data_matrix[i * p + j];
@@ -670,7 +665,7 @@ icainc_JM (float *data_matrix, float *w_matrix, int *nn, int *pp, int *ee,
 
     /* calculate pre-whitening matrix Kmat */
     if (*verbose == 1)	Rprintf ("Whitening\n");
-    Kmat = Calloc (n * n, float);
+    Kmat = Calloc (n * n, double);
     calc_K_JM(data_pre, nn, pp, Kmat);
 
     /* pre-whiten data and reduce dimension from size n to size e */
@@ -683,8 +678,8 @@ icainc_JM (float *data_matrix, float *w_matrix, int *nn, int *pp, int *ee,
     mmult_JM (Kmat1, e, n, data_pre, n, p, data1);
 
     /* calculate initial (orthogonal) unmixing matrix w */
-    temp1 = Calloc (e * e, float);
-    w_init = Calloc (e * e, float);
+    temp1 = Calloc (e * e, double);
+    w_init = Calloc (e * e, double);
     for (i = 0; i < e; i++) {
 	for (j = 0; j < e; j++) {
 	    temp1[i * e + j] = w_matrix[i * e + j];
@@ -730,8 +725,8 @@ icainc_JM (float *data_matrix, float *w_matrix, int *nn, int *pp, int *ee,
     }
 
     if (*defflag == 1) {
-	temp_w1 = Calloc (e, float);
-	temp_w2 = Calloc (e, float);
+	temp_w1 = Calloc (e, double);
+	temp_w2 = Calloc (e, double);
 
 	if (*funflag == 1) {
 	    if (*verbose == 1)
@@ -824,7 +819,7 @@ icainc_JM (float *data_matrix, float *w_matrix, int *nn, int *pp, int *ee,
 
 static const R_CMethodDef CEntries[] = {
     {"icainc_JM", (DL_FUNC) &icainc_JM, 18},
-   {NULL, NULL, 0}
+    {NULL, NULL, 0}
 };
 
 
